Con                                    'S2 RC receive/operate w/encoders triggering leds and audio rec/playb when stopped
                                       'this uses the aileron/elevator stick on an aircraft transmitter
  _CLKMODE = XTAL1 + PLL16X
  _XINFREQ = 5_000_000
  BITS = 14
  
Var
  long buffer[8000]
  
Obj
    
  pwm : "s2motor_driver"
  
Pub Main | aile, elev, left, right, audio, motors
    
  bufflength := 8000
  audio := cognew(@asm_entry, @buffer)  'audio rec/playb cog
  cognew(@entry, 0)                     'encoders/leds cog
  
  repeat
    aile := pulse_in(0)                 'get us pulse width on P0 hacker port from rc receiver
    elev := pulse_in(1)                 'get us pulse width on P1 hacker port from rc receiver
    left := elev - aile                 'elevator stick controls forward/reverse
    right := elev + aile                'aileron stick controls steering/turning
    
    if ||aile < 10                      'create a dead band of +/- 9us
      aile := 0                         'to help with radios maybe a little bit off center trim
    if ||elev < 10                      'that helps when the aile/elev stick is at detent(not touched)
      elev := 0                         'that zero will be returned
      
    if elev <> 0 or aile <> 0           'if the aileron/elevator stick is moved
      if audio                          'shutdown audio cog
        cogstop(audio)
        audio := 0
      if not motors                     'start S2 motor control cog @4khz
        motors := pwm.start(4000)       'pwm control range 0 to -400 full reverse, 0 to 400 full forward
      pwm.leftrightduty(left, right)    'sets the pwm width for each wheel
      
    if elev == 0 and aile == 0          'if aileron/elevator stick is at detent(not touched)
      if motors                         'stop pwm(motors) cog
        pwm.stop
        motors := 0                     
      if not audio                      'start record/playback audio cog
         audio := cognew(@asm_entry, @buffer) 
 
Pub pulse_in(pin) | t0, t1              'get us pulse width from radio receiver plugged into hacker port
                                            
  waitpne(1 << pin, 1 << pin, 0)            
  waitpeq(1 << pin, 1 << pin, 0)            
  t0 := cnt                                 
  waitpne(1 << pin, 1 << pin, 0)            
  t1 := cnt                                 
  return (t1 - t0) / (clkfreq / 1_000_000) - 1500  'return pulse width -1500us so aile/elev stick returns zero when at detent(not touched)

Dat
                      's2 left/right encoders state to green/red left/right leds cog
                        org 0
entry                   
                        andn outa, dirmask             
                        mov dira, dirmask
                        mov time, cnt
                        add time, period
                        mov time1, #0
                        
:loop
                        test lftenc, ina        wc
              if_c      mov lights, #%1001_0001                        
              if_nc     mov lights, #%1010_0001
                        call #leds
                                  
                        waitcnt time, period
                                                                       
                        test rgtenc, ina        wc
              if_c      mov lights, #%1000_0101
              if_nc     mov lights, #%1000_1001
                        call #leds
                        
                        waitcnt time, period
                                   
                        jmp #:loop
                        
leds                         
                        mov countr, #8
:loop                   shr lights, #1          wc 
                        muxc outa, data
                        or outa, clk
                        andn outa, clk
                        djnz countr, #:loop
leds_ret                ret                        
                                      
lftenc        long      1 << 13
rgtenc        long      1 << 14
period        long      1_000                        
data          long      1 << 7
clk           long      1 << 8
dirmask       long      $180
countr        res       1
lights        res       1
time          res       1
time1         res       1
                        fit 496
                        
                      'audio record/playback cog                                                     
                        org 0                               
asm_entry                                        
                        mov       buffbegin, par           
                        mov       dira, asm_dira           
                                                 
                        movs      ctra, #10                
                        movd      ctra, #9                 
                        movi      ctra, #%01001_000        
                        mov       frqa, #1                 
                                                 
                        movs      ctrb, #21                
                        movi      ctrb, #%00110_000        
:init                                            
                        mov       cntr, bufflength         
                        mov       buff, buffbegin
                        mov       tmp, cnt
                        add       tmp, length
                        waitcnt   tmp, length          
                        mov       asm_cnt, cnt             
                        add       asm_cnt, asm_cycles
                                              
:r                      waitcnt   asm_cnt, asm_cycles      
                        mov       asm_sample, phsa         
                        sub       asm_sample, asm_old      
                        add       asm_old, asm_sample      
                        shl       asm_sample, #32-BITS     
                        wrlong    asm_sample, buff         
                        add       buff, #4                 
                        djnz      cntr, #:r                
                                             
                        mov       buff, buffbegin       
                        mov       cntr, bufflength 
                        mov       asm_cnt, cnt             
                        add       asm_cnt, asm_cycles      
:p                                            
                        rdlong    frqb, buff               
                        add       buff, #4                 
                        waitcnt   asm_cnt, asm_cycles      
                        djnz      cntr, #:p                
                        jmp       #:init                   
                                                 
asm_cycles    long      |< BITS - 1              
asm_dira      long      $200200                  
bufflength    long      0
length        long      $0262_5a00                        
asm_cnt       res       1                        
asm_old       res       1                        
asm_sample    res       1                        
buff          res       1                        
buffbegin     res       1                        
cntr          res       1
tmp           res       1
                        fit 496             